#!/bin/bash
#

# if this build script complains that it cannot find bzlib.h,
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e10|e14>:<[mpich|mvapich2]> <prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir()
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`

    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi
    qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
    (( seen_bqual = 0 ))
    for qual in "${qlist[@]}"; do
        case $qual in
            e[1-9]*)
                cqual=$qual
                ;;
            mpich|mvapich2)
                (( seen_bqual++ )) && ssi_die "Qualifiers mpich and mvapich2 are mutually exclusive"
                bqual=$qual
                ;;
            *)
                ssi_die "Unrecognized qualifier $qual"
        esac
    done

    if [[ -z "$cqual" ]]; then
        ssi_die "Base qualifier is mandatory"
    fi

    if [[ -z "${extraqual}" ]] && [[ -n "${bqual}" ]]; then
      ssi_die "build type is mandatory."
    fi
    basequal=${cqual}${bqual:+:${bqual}}
 }

set_compiler_flags()
{
    case ${cqual} in
        e[79]|e1[04])
            :
            ;;
        *)
            ssi_die "Unrecognized compiler qualifier $cqual"
    esac
    [[ -n "${extraqual}" ]] && 
    case $extraqual in
        debug|opt)
            ssi_die "$extraqual build is not supported for this package."
            ;;
        prof)
            reflag=RelWithDebInfo
            ;;
        *)
            ssi_die "Unrecognized extra qualifier $extraqual"
    esac
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
if [[ "$extraqual" == "tar" ]]; then
  unset extraqual
  maketar=tar
fi

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=hep_hpc
origpkgver=v0_01_00
pkgver=${origpkgver//-/_}
ssibuildshims_version=v0_17_00
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=${package}-${pkgdotver}.tar.bz2

get_this_dir
get_ssibuildshims

set -- # Clear positional parameters.

source ssi_functions # For ssi_die().

set_qualifiers
source define_basics $define_basics_args
set_compiler_flags

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "fake setup failed"

ups active

set -x

cd "${sourcedir}" && rm -rf ${pkgtardir} || ssi_die "Unable to cd into ${sourcedir}"
tar xf ${tardir}/${pkgtarname}* || ssi_die "Unable to untar source"

cd "${blddir}" || ssi_die "Unable to cd into ${blddir}"

eval CC=${cc-gcc} CXX=${cxx-g++} \
  cmake -DCMAKE_BUILD_TYPE=${relflag} \
  -DCMAKE_INSTALL_PREFIX=${pkgdir} \
  ${sourcedir}/* || \
  ssi_die "configure failed"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

make -j ${ncore} || ssi_die "make failed"

make install || ssi_die "make install failed"

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  ssi_die "failed to declare ${package} ${pkgver} -q ${fullqual}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "$package is installed at $(eval echo "\${${package}_FQ_DIR}")"


# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
